import os
import platform
import signal
import subprocess
import sys
from textwrap import dedent
from pathlib import Path

import colorama


def main():
    colorama.init()

    cwd = Path().resolve()
    name = ".venv"
    venv_folder = find_venv(cwd, name)
    if venv_folder is None:
        error(
            f"""
            Cannot start python-localvenv kernel:

            Expected folder "{name}" in notebook directory
            {cwd}
            (or any parent directory)
            """
        )
    python = venv_folder / "bin" / "python"
    if not is_exe(python):
        error(
            f"""
            Cannot start python-localvenv kernel:

            {python}
            for launching environment kernel is not executable
            """
        )
    cmd_check_kernel = [
        python,
        "-m",
        "ipykernel_launcher",
        "--version",
    ]
    try:
        kernel_version = subprocess.check_output(
            cmd_check_kernel, stderr=subprocess.STDOUT, text=True
        )
    except subprocess.CalledProcessError as exc_info:
        error(
            f"""
            Cannot start python-localvenv kernel:

            {' '.join([str(a) for a in cmd_check_kernel])}
            returned exit status {exc_info.returncode}

            ERROR: {exc_info.output.strip()}

            Make sure that the 'ipykernel' package is installed in the virtual
            environment {venv_folder}
            """
        )
    cmd = [
        python,
        "-m",
        "ipykernel_launcher",
        *sys.argv[1:],
    ]
    print(
        colorama.Fore.GREEN
        + "PYTHON-LOCALVENV KERNEL"
        + colorama.Style.RESET_ALL
        + " delegate to "
        + " ".join([str(part) for part in cmd])
        + f" (version {kernel_version.strip()})",
        file=sys.stderr,
    )
    proc = subprocess.Popen(cmd)

    if platform.system() == "Windows":
        forward_signals = set(signal.Signals) - {
            signal.CTRL_BREAK_EVENT,
            signal.CTRL_C_EVENT,
            signal.SIGTERM,
        }
    else:
        forward_signals = set(signal.Signals) - {
            signal.SIGKILL,
            signal.SIGSTOP,
        }

    def handle_signal(sig, _frame):
        proc.send_signal(sig)

    for sig in forward_signals:
        signal.signal(sig, handle_signal)

    exit_code = proc.wait()
    if exit_code == 0:
        print(
            "PYTHON-LOCALVENV KERNEL: ipykernel_launcher exited",
            file=sys.stderr,
        )
    else:
        print(
            "PYTHON-LOCALVENV KERNEL: ipykernel_launcher exited with code:",
            exit_code,
            file=sys.stderr,
        )


def is_exe(fpath):
    return os.path.isfile(fpath) and os.access(fpath, os.X_OK)


def error(msg):
    print(
        colorama.Fore.RED
        + colorama.Style.BRIGHT
        + "\n"
        + "!" * 80
        + dedent(msg)
        + "!" * 80
        + "\n"
        + colorama.Style.RESET_ALL,
        file=sys.stderr,
    )
    sys.exit(1)


def find_venv(root, name):
    candidate_dirs = [root, *root.parents]
    for dirs in candidate_dirs:
        venv_folder = dirs / name
        if venv_folder.is_dir():
            return venv_folder
    return None


if __name__ == "__main__":
    main()
